import os
from pyramid.testing import DummyRequest
from pyramid.compat import text_
from pyramid.httpexceptions import HTTPFound

import ptah
from ptah.authentication import AuthInfo

import ptah_crowd


class Principal(object):

    def __init__(self, uri, name, login):
        self.__uri__ = uri
        self.name = name
        self.login = login


class TestValidation(ptah.PtahTestCase):

    def test_validation_auth_checker_validation(self):
        from ptah_crowd.validation import validationAndSuspendedChecker

        principal = Principal('1', 'user', 'user')

        props = ptah_crowd.get_properties(principal.__uri__)
        props.validated = False

        cfg = ptah.get_settings(ptah_crowd.CFG_ID_CROWD)

        # validation disabled
        info = AuthInfo(principal, True)
        cfg['validation'] = False
        self.assertTrue(validationAndSuspendedChecker(info))

        info = AuthInfo(principal, True)
        cfg['allow-unvalidated'] = False
        self.assertTrue(validationAndSuspendedChecker(info))

        # validation enabled
        info = AuthInfo(principal, True)
        cfg['validation'] = True
        cfg['allow-unvalidated'] = False
        self.assertFalse(validationAndSuspendedChecker(info))
        self.assertEqual(info.message, 'Account is not validated.')

        info = AuthInfo(principal, True)
        cfg['allow-unvalidated'] = True
        self.assertTrue(validationAndSuspendedChecker(info))

        # validated
        props = ptah_crowd.get_properties(principal.__uri__)
        props.validated = True

        info = AuthInfo(principal, True)
        cfg['validation'] = True
        self.assertTrue(validationAndSuspendedChecker(info))

    def test_validation_auth_checker_suspended(self):
        from ptah.authentication import AuthInfo
        from ptah_crowd.validation import validationAndSuspendedChecker

        principal = Principal('2', 'user', 'user')

        props = ptah_crowd.get_properties(principal.__uri__)
        props.validated = True
        props.suspended = False

        info = AuthInfo(principal, True)
        self.assertTrue(validationAndSuspendedChecker(info))

        props.suspended = True

        info = AuthInfo(principal, True)
        self.assertFalse(validationAndSuspendedChecker(info))
        self.assertEqual(info.message, 'Account is suspended.')

    def test_validation_registered_unvalidated(self):
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login', email='email')
        CrowdFactory().add(user)

        cfg = ptah.get_settings(ptah_crowd.CFG_ID_CROWD)
        cfg['validation'] = True
        self.registry.notify(ptah.events.PrincipalRegisteredEvent(user))

        props = ptah_crowd.get_properties(user.__uri__)
        self.assertFalse(props.validated)

    def test_validation_registered_no_validation(self):
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login', email='email')
        CrowdFactory().add(user)

        cfg = ptah.get_settings(ptah_crowd.CFG_ID_CROWD)
        cfg['validation'] = False
        self.registry.notify(ptah.events.PrincipalRegisteredEvent(user))

        props = ptah_crowd.get_properties(user.__uri__)
        self.assertTrue(props.validated)

    def test_validation_added(self):
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login', email='email')
        CrowdFactory().add(user)

        cfg = ptah.get_settings(ptah_crowd.CFG_ID_CROWD)
        cfg['validation'] = False
        self.registry.notify(ptah.events.PrincipalAddedEvent(user))

        props = ptah_crowd.get_properties(user.__uri__)
        self.assertTrue(props.validated)

        user = CrowdUser(title='name', login='login', email='email')
        cfg['validation'] = True
        self.registry.notify(ptah.events.PrincipalAddedEvent(user))

        props = ptah_crowd.get_properties(user.__uri__)
        self.assertTrue(props.validated)

    def test_validation_initiate(self):
        from ptah_crowd import validation
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        origValidationTemplate = validation.ValidationTemplate

        class Stub(origValidationTemplate):

            status = ''
            token = None

            def send(self):
                Stub.status = 'Email has been sended'
                Stub.token = self.token

        validation.ValidationTemplate = Stub

        user = CrowdUser(title='name', login='login', email='email')
        CrowdFactory().add(user)

        validation.initiate_email_validation(user.email, user, self.request)
        self.assertEqual(Stub.status, 'Email has been sended')
        self.assertIsNotNone(Stub.token)

        t = ptah.token.service.get_bydata(validation.TOKEN_TYPE, user.__uri__)
        self.assertEqual(Stub.token, t)

        validation.ValidationTemplate = origValidationTemplate

    def test_validation_template(self):
        from ptah_crowd import validation
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        origValidationTemplate = validation.ValidationTemplate
        user = CrowdUser(title='name', login='login', email='email')
        CrowdFactory().add(user)

        template = validation.ValidationTemplate(
            user, self.request, email = user.email, token = 'test-token')
        template.update()

        res = text_(template.render())

        self.assertIn(
            "You're close to completing the registration process.", res)
        self.assertIn(
            "http://example.com/validateaccount.html?token=test-token", res)

    def test_validate(self):
        from ptah_crowd import validation
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login', email='email')
        CrowdFactory().add(user)

        t = ptah.token.service.generate(validation.TOKEN_TYPE, user.__uri__)

        res = validation.validate(self.request)
        self.assertIsInstance(res, HTTPFound)
        self.assertIn(
            "Can't validate email address.", ptah.render_messages(self.request))

        props = ptah_crowd.get_properties(user.__uri__)
        props.validated = False
        self.request.GET['token'] = t
        self.request.session.clear()

        res = validation.validate(self.request)
        self.assertIn("Account has been successfully validated.",
                      ptah.render_messages(self.request))

        props = ptah_crowd.get_properties(user.__uri__)
        self.assertTrue(props.validated)



""" login form """
import ptah
from ptah import view, form
from pyramid import security
from pyramid.view import view_config
from pyramid.httpexceptions import HTTPFound

from ptah_crowd.settings import _, CFG_ID_CROWD
from ptah_crowd.memberprops import get_properties


@view_config(
    route_name='ptah-login',
    wrapper=ptah.wrap_layout('ptah-page'),
    renderer="ptah_crowd:templates/login.pt")

class LoginForm(form.Form):
    """ Login form """

    id = 'login-form'
    title = _('Login')

    fields = form.Fieldset(
        form.fields.TextField(
            'login',
            title = _('Login Name'),
            description = _('Login names are case sensitive, '\
                                'make sure the caps lock key is not enabled.'),
            default = ''),

        form.fields.PasswordField(
            'password',
            title = _('Password'),
            description = _('Case sensitive, make sure caps '\
                                'lock is not enabled.'),
            default = ''),
        )

    @form.button(_("Log in"), name='login', actype=form.AC_PRIMARY)
    def handleLogin(self):
        request = self.request

        data, errors = self.extract()
        if errors:
            self.message(errors, 'form-error')
            return

        info = ptah.auth_service.authenticate(data)

        if info.status:
            request.registry.notify(
                ptah.events.LoggedInEvent(info.principal))

            location = request.application_url

            came_from = request.GET.get('came_from', '')
            if came_from.startswith(location):
                location = came_from
            else:
                location = '%s/login-success.html'%location

            headers = security.remember(request, info.__uri__)
            return HTTPFound(headers = headers, location = location)

        if info.principal is not None:
            request.registry.notify(
                ptah.events.LoginFailedEvent(info.principal, info.message))

        if info.arguments.get('suspended'):
            return HTTPFound(
                location='%s/login-suspended.html'%request.application_url)

        if info.message:
            self.message(info.message, 'warning')
            return

        self.message(_('You enter wrong login or password.'), 'error')

    def update(self):
        self.app_url = self.request.application_url
        cfg = ptah.get_settings(CFG_ID_CROWD, self.request.registry)
        self.join = cfg['join']
        if cfg['joinurl']:
            self.joinurl = cfg['joinurl']
        else:
            self.joinurl = '%s/join.html'%self.app_url

        if ptah.auth_service.get_userid():
            return HTTPFound(location = '%s/login-success.html'%self.app_url)

        return super(LoginForm, self).update()


@view_config(
    route_name='ptah-login-success', wrapper=ptah.wrap_layout('ptah-page'),
    renderer='ptah_crowd:templates/login-success.pt')

class LoginSuccess(ptah.View):
    """ Login successful information page. """

    def update(self):
        user = ptah.auth_service.get_current_principal()
        if user is None:
            request = self.request
            headers = security.forget(request)

            return HTTPFound(
                headers = headers,
                location = '%s/login.html'%request.application_url)
        else:
            self.user = user.name or user.login


@view_config(
    route_name='ptah-login-suspended', wrapper=ptah.wrap_layout('ptah-page'),
    renderer="ptah_crowd:templates/login-suspended.pt")

class LoginSuspended(ptah.View):
    """ Suspended account information page. """

    def update(self):
        uid = ptah.auth_service.get_userid()
        if not uid:
            return HTTPFound(location=self.request.application_url)

        props = get_properties(uid)
        if not props.suspended:
            return HTTPFound(location=self.request.application_url)

        MAIL = ptah.get_settings(ptah.CFG_ID_PTAH)
        self.from_name = MAIL['email_from_name']
        self.from_address = MAIL['email_from_address']
        self.full_address = MAIL['full_email_address']


@view_config(route_name='ptah-logout')
def logout(request):
    """Logout action"""
    uid = ptah.auth_service.get_userid()

    if uid is not None:
        ptah.auth_service.set_userid(None)
        request.registry.notify(
            ptah.events.LoggedOutEvent(ptah.resolve(uid)))

        view.add_message(request, _('Logout successful!'), 'info')
        headers = security.forget(request)
        return HTTPFound(
            headers = headers,
            location = request.application_url)
    else:
        return HTTPFound(location = request.application_url)



""" member properties """
import ptah
import sqlalchemy as sqla
from datetime import datetime


def query_properties(uri):
    return MemberProperties._sql_get.first(uri=uri)


def get_properties(uri):
    props = MemberProperties._sql_get.first(uri=uri)
    if props is None:
        props = MemberProperties(uri)
        Session = ptah.get_session()
        Session.add(props)
        Session.flush()
    return props


class MemberProperties(ptah.get_base()):

    __tablename__ = 'ptah_crowd_memberprops'

    uri = sqla.Column(sqla.Unicode, primary_key=True, info={'uri': True})
    joined = sqla.Column(sqla.DateTime())
    validated = sqla.Column(sqla.Boolean, default=False)
    suspended = sqla.Column(sqla.Boolean(), default=False)
    data = sqla.Column(ptah.JsonDictType(), default={})

    def __init__(self, uri):
        super(MemberProperties, self).__init__()

        self.uri = uri
        self.joined = datetime.now()
        self.validated = False
        self.suspended = False

    _sql_get = ptah.QueryFreezer(
        lambda: ptah.get_session().query(MemberProperties)
        .filter(MemberProperties.uri == sqla.sql.bindparam('uri')))



# ptah_crowd package

from ptah_crowd.provider import CrowdUser
from ptah_crowd.provider import CrowdApplication
from ptah_crowd.provider import CrowdFactory
from ptah_crowd.provider import CROWD_APP_ID

from ptah_crowd.settings import CFG_ID_CROWD
from ptah_crowd.memberprops import get_properties
from ptah_crowd.memberprops import query_properties
from ptah_crowd.validation import initiate_email_validation

from ptah_crowd.schemas import UserSchema
from ptah_crowd.schemas import checkLoginValidator


# ptah_crowd include
def includeme(config):
    config.add_route(
        'ptah-principal-validate', '/validateaccount.html')

    # login routes
    config.add_route(
        'ptah-login', '/login.html', use_global_views=True)
    config.add_route(
        'ptah-logout', '/logout.html', use_global_views=True)
    config.add_route(
        'ptah-login-success', '/login-success.html', use_global_views=True)
    config.add_route(
        'ptah-login-suspended', '/login-suspended.html', use_global_views=True)

    # reset password
    config.add_route(
        'ptah-resetpassword', '/resetpassword.html')
    config.add_route(
        'ptah-resetpassword-form', '/resetpassword.html/*subpath')

    # registration
    config.add_route(
        'ptah-join', '/join.html', use_global_views=True)

    # for management module
    config.add_route(
        CROWD_APP_ID, '# {0}'.format(CROWD_APP_ID), use_global_views=True)

    # scan
    config.scan('ptah_crowd')



""" add/edit user """
from zope import interface
from ptah import config, view, form
from pyramid.view import view_config
from pyramid.httpexceptions import HTTPFound

import ptah
import ptah_crowd
from ptah_crowd.settings import _
from ptah_crowd.module import CrowdModule
from ptah_crowd.provider import CrowdUser, CrowdFactory
from ptah_crowd.schemas import UserSchema, ManagerChangePasswordSchema


@view_config('create.html',
             context=CrowdModule,
             wrapper=ptah.wrap_layout())

class CreateUserForm(form.Form):

    csrf = True
    label = _('Create new user')
    fields = UserSchema

    @form.button(_('Create'), actype=form.AC_PRIMARY)
    def create(self):
        data, errors = self.extract()

        if errors:
            self.message(errors, 'form-error')
            return

        # create user
        cfg = ptah.get_settings(ptah_crowd.CFG_ID_CROWD, self.request.registry)
        tp = cfg['type']
        if not tp.startswith('cms-type:'):
            tp = 'cms-type:{0}'.format(tp)

        tinfo = ptah.resolve(tp)

        user = tinfo.create(
            title=data['name'], login=data['login'], email=data['login'])
        user.password = ptah.pwd_tool.encode(data['password'])

        crowd = CrowdFactory()
        crowd.add(user)

        # notify system
        self.request.registry.notify(ptah.events.PrincipalAddedEvent(user))

        # set props
        props = ptah_crowd.get_properties(user.__uri__)
        props.validated = data['validated']
        props.suspended = data['suspended']

        self.message('User has been created.', 'success')
        return HTTPFound(location='.')

    @form.button(_('Back'))
    def back(self):
        return HTTPFound(location='.')


@view_config(context=CrowdUser,
             wrapper=ptah.wrap_layout(),
             route_name=ptah_crowd.CROWD_APP_ID)

class ModifyUserForm(form.Form):

    csrf = True
    label = 'Update user'
    fields = form.Fieldset(UserSchema)

    def form_content(self):
        user = self.context
        props = ptah_crowd.get_properties(user.__uri__)

        return {'name': user.name,
                'login': user.login,
                'password': '',
                'validated': props.validated,
                'suspended': props.suspended}

    @form.button(_('Modify'), actype=form.AC_PRIMARY)
    def modify(self):
        data, errors = self.extract()

        if errors:
            self.message(errors, 'form-error')
            return

        user = self.context

        # update attrs
        user.title = data['name']
        user.login = data['login']
        user.email = data['login']
        user.password = ptah.pwd_tool.encode(data['password'])

        # update props
        props = ptah_crowd.get_properties(user.__uri__)
        props.validated = data['validated']
        props.suspended = data['suspended']

        self.message("User properties has been updated.", 'info')

    @form.button(_('Remove'), actype=form.AC_DANGER)
    def remove(self):
        self.validate_csrf_token()

        user = self.context
        Session = ptah.get_session()
        Session.delete(user)
        Session.flush()

        self.message("User has been removed.", 'info')
        return HTTPFound(location='..')

    @form.button(_('Change password'), name='changepwd')
    def password(self):
        return HTTPFound(location='password.html')

    @form.button(_('Back'))
    def back(self):
        return HTTPFound(location='..')


@view_config('password.html',
             context=CrowdUser,
             wrapper=ptah.wrap_layout(),
             route_name=ptah_crowd.CROWD_APP_ID)

class ChangePasswordForm(form.Form):

    csrf = True
    fields = form.Fieldset(ManagerChangePasswordSchema)

    label = _('Change password')
    description = _('Please specify password for this users.')

    @form.button(_('Change'), actype=form.AC_PRIMARY)
    def change(self):
        data, errors = self.extract()

        if errors:
            self.message(errors, 'form-error')
            return

        sm = self.request.registry

        self.context.password = ptah.pwd_tool.encode(data['password'])

        self.message("User password has been changed.")

    @form.button(_('Back'))
    def back(self):
        return HTTPFound(location='..')



import sqlalchemy as sqla
from pyramid.config import Configurator
from pyramid.events import ApplicationCreated

import ptah
from ptah.password import ID_PASSWORD_CHANGER
from ptah_crowd.settings import CFG_ID_CROWD
from ptah_crowd.memberprops import get_properties

CROWD_APP_ID = 'ptah-crowd'


class CrowdUser(ptah.cms.BaseContent):

    __tablename__ = 'ptah_crowd'

    __type__ = ptah.cms.Type(
        'ptah-crowd-user', 'Crowd user', global_allow = False)

    login = sqla.Column(sqla.Unicode(255), unique=True)
    email = sqla.Column(sqla.Unicode(255), unique=True)
    password = sqla.Column(sqla.Unicode(255))

    @property
    def name(self):
        return self.title

    @property
    def properties(self):
        return get_properties(self.__uri__)

    def __str__(self):
        return self.name

    def __repr__(self):
        return '%s<%s:%s>'%(self.__class__.__name__, self.title, self.__uri__)


def get_allowed_content_types(context, registry=None):
    CROWD = ptah.get_settings(CFG_ID_CROWD, registry)
    return (CROWD['type'],)


class CrowdApplication(ptah.cms.BaseApplicationRoot,ptah.cms.BaseContainer):

    __type__ = ptah.cms.Type('ptah-crowd-provider',
                             'Ptah user management',
                             filter_content_types = True,
                             allowed_content_types = get_allowed_content_types)

    def add(self, user):
        Session = ptah.get_session()
        Session.add(user)
        Session.flush()

        self[str(user.__id__)] = user


@ptah.auth_provider('ptah-crowd-auth')
class CrowdAuthProvider(object):

    _sql_get_login = ptah.QueryFreezer(
        lambda: ptah.get_session().query(CrowdUser)\
            .filter(CrowdUser.login==sqla.sql.bindparam('login')))

    _sql_search = ptah.QueryFreezer(
        lambda: ptah.get_session().query(CrowdUser) \
            .filter(sqla.sql.or_(
                CrowdUser.email.contains(sqla.sql.bindparam('term')),
                CrowdUser.title.contains(sqla.sql.bindparam('term'))))\
            .order_by(sqla.sql.asc('title')))

    def authenticate(self, creds):
        login, password = creds['login'], creds['password']

        user = self._sql_get_login.first(login=login)
        if user is not None:
            if ptah.pwd_tool.check(user.password, password):
                return user

    def get_principal_bylogin(self, login):
        return self._sql_get_login.first(login=login)

    @classmethod
    def search(cls, term):
        for user in cls._sql_search.all(term = '%%%s%%'%term):
            yield user

    @staticmethod
    def change_password(principal, password):
        principal.password = password


CrowdFactory = ptah.cms.ApplicationFactory(
    CrowdApplication,
    name = CROWD_APP_ID,
    title = 'Ptah user management')


@ptah.subscriber(ApplicationCreated)
def initialize(ev):
    cfg = Configurator(ev.app.registry, autocommit=True)

    settings = ptah.get_settings(CFG_ID_CROWD, ev.app.registry)

    tp = settings['type']
    if not tp.startswith('cms-type:'):
        tp = 'cms-type:{0}'.format(tp)

    tinfo = ptah.resolve(tp)
    if tinfo is not None:
        if tinfo.schema not in ptah.get_cfg_storage(ID_PASSWORD_CHANGER):
            cfg.ptah_password_changer(
                tinfo.schema, CrowdAuthProvider.change_password)
            cfg.ptah_principal_searcher(
                tinfo.schema, CrowdAuthProvider.search)



import transaction
import ptah
from ptah import config
from ptah.testing import PtahTestCase
from pyramid.testing import DummyRequest
from pyramid.httpexceptions import HTTPException, HTTPFound, HTTPForbidden

import ptah_crowd


class TestCreateUser(PtahTestCase):

    def test_create_user_back(self):
        from ptah_crowd.module import CrowdModule
        from ptah_crowd.user import CreateUserForm

        request = DummyRequest(
            POST = {'form.buttons.back': 'Back'})
        mod = CrowdModule(None, request)

        view = CreateUserForm(mod, request)
        res = view.update()
        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], '.')

    def test_create_user_error(self):
        from ptah_crowd.module import CrowdModule
        from ptah_crowd.user import CreateUserForm

        f = CreateUserForm(None, None)

        request = DummyRequest(
            POST = {'form.buttons.create': 'Create'})
        request.POST[CreateUserForm.csrfname] = \
            request.session.get_csrf_token()
        mod = CrowdModule(None, request)

        view = CreateUserForm(mod, request)
        view.update()
        self.assertIn(
            'Please fix indicated errors.',
            ptah.render_messages(request))

    def test_create_user(self):
        from ptah_crowd.module import CrowdModule
        from ptah_crowd.user import CreateUserForm

        f = CreateUserForm(None, None)

        request = DummyRequest(
            POST = {'name': 'NKim',
                    'login': 'ptah@ptahproject.org',
                    'password': '12345',
                    'validated': 'false',
                    'suspended': 'true',
                    'form.buttons.create': 'Create'})
        request.POST[CreateUserForm.csrfname] = \
            request.session.get_csrf_token()

        mod = CrowdModule(None, request)

        view = CreateUserForm(mod, request)
        res = view.update()

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], '.')
        self.assertIn(
            'User has been created.', ptah.render_messages(request))

        user = ptah.auth_service.get_principal_bylogin('ptah@ptahproject.org')
        self.assertEqual(user.name, 'NKim')
        self.assertEqual(user.login, 'ptah@ptahproject.org')

        props = ptah_crowd.query_properties(user.__uri__)
        self.assertTrue(props.suspended)
        self.assertFalse(props.validated)


class TestModifyUser(PtahTestCase):

    def _user(self):
        from ptah_crowd.provider import CrowdUser, CrowdFactory
        user = CrowdUser(title='name', login='ptah@local', email='ptah@local')
        CrowdFactory().add(user)
        return user

    def test_modify_user_back(self):
        from ptah_crowd.user import ModifyUserForm

        user = self._user()

        request = DummyRequest(
            POST = {'form.buttons.back': 'Back'})

        view = ModifyUserForm(user, request)
        res = view.update()

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], '..')

    def test_modify_user_changepwd(self):
        from ptah_crowd.user import ModifyUserForm

        user = self._user()

        request = DummyRequest(
            POST = {'form.buttons.changepwd': 'Change'})

        view = ModifyUserForm(user, request)
        res = view.update()

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], 'password.html')

    def test_modify_user_forbidden(self):
        from ptah_crowd.user import ModifyUserForm

        user = self._user()

        request = DummyRequest(
            POST = {'form.buttons.modify': 'Modify',
                    'name': 'NKim',
                    'login': 'ptah@ptahproject.org',
                    'password': '12345',
                    'validated': 'false',
                    'suspended': 'true',
                    })

        view = ModifyUserForm(user, request)
        res = None
        try:
            view.update()
        except Exception as err:
            res = err

        self.assertIsInstance(res, HTTPForbidden)
        self.assertEqual(str(res), 'Form authenticator is not found.')

    def test_modify_user_error(self):
        from ptah_crowd.user import ModifyUserForm

        user = self._user()

        request = DummyRequest(
            POST = {'form.buttons.modify': 'Modify'})

        view = ModifyUserForm(user, request)
        view.csrf = False
        view.update()

        self.assertIn(
            'Please fix indicated errors.',
            ptah.render_messages(request))

    def test_modify_user(self):
        from ptah_crowd.user import ModifyUserForm

        user = self._user()
        f = ModifyUserForm(None, None)

        request = DummyRequest(
            POST = {'form.buttons.modify': 'Modify',
                    'name': 'NKim',
                    'login': 'ptah@ptahproject.org',
                    'password': '12345',
                    'validated': 'false',
                    'suspended': 'true'})

        view = ModifyUserForm(user, request)
        view.csrf = False
        view.update()

        self.assertEqual(user.name, 'NKim')
        self.assertEqual(user.login, 'ptah@ptahproject.org')

    def test_modify_user_remove(self):
        from ptah_crowd.user import ModifyUserForm
        from ptah_crowd.provider import CrowdUser

        user = self._user()

        request = DummyRequest(
            POST = {'form.buttons.remove': 'Remove'})

        view = ModifyUserForm(user, request)
        view.csrf = False
        res = view.update()

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], '..')

        user = ptah.resolve(user.__uri__)
        self.assertIsNone(user)


class TestChangePassword(PtahTestCase):

    def _user(self):
        from ptah_crowd.provider import CrowdUser, CrowdFactory
        user = CrowdUser(title='name', login='ptah@local', email='ptah@local')
        CrowdFactory().add(user)
        return user

    def test_change_password_user_back(self):
        from ptah_crowd.user import ChangePasswordForm

        user = self._user()

        request = DummyRequest(
            POST = {'form.buttons.back': 'Back'})

        view = ChangePasswordForm(user, request)
        res = view.update()

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], '..')

    def test_change_password_forbidden(self):
        from ptah_crowd.user import ChangePasswordForm

        user = self._user()

        request = DummyRequest(
            POST = {'form.buttons.change': 'Change',
                    'password': '12345',
                    })

        view = ChangePasswordForm(user, request)
        res = None
        try:
            view.update()
        except Exception as err:
            res = err

        self.assertIsInstance(res, HTTPForbidden)
        self.assertEqual(str(res), 'Form authenticator is not found.')

    def test_change_password_error(self):
        from ptah_crowd.user import ChangePasswordForm

        user = self._user()
        f = ChangePasswordForm(None, None)

        request = DummyRequest(
            POST = {'form.buttons.change': 'Change'})

        view = ChangePasswordForm(user, request)
        view.csrf = False
        view.update()

        self.assertIn(
            'Please fix indicated errors.', ptah.render_messages(request))

    def test_change_password(self):
        from ptah_crowd.user import ChangePasswordForm

        user = self._user()
        f = ChangePasswordForm(None, None)

        request = DummyRequest(
            POST = {'form.buttons.change': 'Change',
                    'password': '12345'})

        request.POST[ChangePasswordForm.csrfname] = \
                                             request.session.get_csrf_token()

        view = ChangePasswordForm(user, request)
        view.update()

        self.assertEqual(user.password, '{plain}12345')



import ptah
import translationstring

_ = translationstring.TranslationStringFactory('ptah_crowd')


CFG_ID_CROWD = 'ptah-crowd'

ptah.register_settings(
    CFG_ID_CROWD,

    ptah.form.TextField(
        'type',
        title = 'User content type',
        description = 'User content type for crowd user provider',
        default = 'ptah-crowd-user'),

    ptah.form.BoolField(
        'join',
        title = 'Site registration',
        description = 'Enable/Disable site registration',
        default = True),

    ptah.form.TextField(
        'joinurl',
        title = 'Join form url',
        default = ''),

    ptah.form.BoolField(
        'password',
        title = 'User password',
        description = 'Allow use to select password during registration',
        default = False),

    ptah.form.BoolField(
        'validation',
        title = 'Email validation',
        description = 'Validate user account by email.',
        default = True),

    ptah.form.BoolField(
        'allow-unvalidated',
        title = 'Allow un validation',
        description = 'Allow login for un Validated users.',
        default = True),

    title = 'Ptah crowd settings',
    #ttw = True,
    #ttw_skip_fields = ('type',),
    )



import datetime
import transaction

import ptah
import ptah_crowd


class TestMemberprops(ptah.PtahTestCase):

    def test_memberprops_get(self):
        from ptah_crowd.memberprops import MemberProperties

        props = ptah_crowd.get_properties('uid1')

        self.assertIsInstance(props, MemberProperties)
        self.assertEqual(props.uri, 'uid1')
        self.assertEqual(props.validated, False)
        self.assertEqual(props.suspended, False)
        self.assertIsInstance(props.joined, datetime.datetime)

        props = ptah_crowd.get_properties('uid1')
        self.assertEqual(props.uri, 'uid1')

        self.assertEqual(
            ptah.get_session().query(MemberProperties).count(), 1)

    def test_memberprops_query(self):
        from ptah_crowd.memberprops import MemberProperties

        props = ptah_crowd.query_properties('uid1')
        self.assertIsNone(props)
        self.assertEqual(
            ptah.get_session().query(MemberProperties).count(), 0)

    def test_user_props(self):
        from ptah_crowd import CrowdUser
        from ptah_crowd.memberprops import MemberProperties

        user = CrowdUser(title='user-name', login='user-login',
                         email='user-email', password='passwd')

        self.assertIs(
            user.properties, ptah_crowd.query_properties(user.__uri__))



import transaction
import ptah
from ptah import config
from ptah.testing import PtahTestCase
from webob.multidict import MultiDict
from pyramid.testing import DummyRequest
from pyramid.view import render_view_to_response
from pyramid.httpexceptions import HTTPFound, HTTPForbidden

import ptah_crowd


class TestModule(PtahTestCase):

    def test_manage_module(self):
        from ptah.manage.manage import PtahManageRoute
        from ptah_crowd.module import CrowdModule

        request = DummyRequest()

        ptah.auth_service.set_userid('test')

        cfg = ptah.get_settings(ptah.CFG_ID_PTAH)
        cfg['managers'] = ('*',)
        mr = PtahManageRoute(request)
        mod = mr['crowd']

        self.assertIsInstance(mod, CrowdModule)

    def test_manage_module_get(self):
        from ptah_crowd.module import CrowdModule
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login', email='email')
        CrowdFactory().add(user)

        uri = user.__uri__

        mod = CrowdModule(None, DummyRequest())

        self.assertRaises(KeyError, mod.__getitem__, 'unkown')

        wu = mod[str(user.__id__)]

        self.assertIsInstance(wu, CrowdUser)
        self.assertEqual(wu.__uri__, uri)


class TestModuleView(PtahTestCase):

    def _make_mod(self):
        from ptah_crowd.module import CrowdModule
        return CrowdModule(None, DummyRequest())

    def _make_user(self):
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login', email='email')
        CrowdFactory().add(user)
        return user

    def test_module_search(self):
        from ptah_crowd.views import CrowdApplicationView

        mod = self._make_mod()

        form = CrowdApplicationView(mod, DummyRequest(
                POST=MultiDict((('form.buttons.search', 'Search'),
                                ('term', 'search term')))))
        form.csrf = False
        res = form.update()

        self.assertIn('ptah-search-term', form.request.session)
        self.assertEqual(
            form.request.session['ptah-search-term'], 'search term')

    def test_module_clear(self):
        from ptah_crowd.views import CrowdApplicationView

        mod = self._make_mod()

        form = CrowdApplicationView(mod, DummyRequest(
                session = {'ptah-search-term': 'test'},
                POST=MultiDict((('form.buttons.clear', 'Clear'),))))
        form.csrf = False
        form.update()

        self.assertNotIn('ptah-search-term', form.request.session)

    def test_module_search_error(self):
        from ptah_crowd.views import CrowdApplicationView

        mod = self._make_mod()

        form = CrowdApplicationView(mod, DummyRequest(
                POST=MultiDict((('form.buttons.search', 'Search'),))))
        form.csrf = False
        form.update()

        self.assertIn('Please specify search term',
                      ptah.render_messages(form.request))

    def test_module_list(self):
        from ptah_crowd.views import CrowdApplicationView

        mod = self._make_mod()
        user = self._make_user()

        request = DummyRequest(
            params = MultiDict(), POST = MultiDict())
        request.session['ptah-search-term'] = 'email'

        res = render_view_to_response(mod, request, '')

        self.assertIn('value="%s"'%user.__uri__, res.text)

        res = render_view_to_response(
            mod,
            DummyRequest(params = MultiDict(), POST = MultiDict()), '')

        self.assertIn('value="%s"'%user.__uri__, res.text)

        res = render_view_to_response(
            mod,
            DummyRequest(params = MultiDict({'batch': 1}),
                         POST = MultiDict()), '')

        self.assertIn('value="%s"'%user.__uri__, res.text)

        res = render_view_to_response(
            mod,
            DummyRequest(params = MultiDict({'batch': 0}),
                         POST = MultiDict()), '')

        self.assertIn('value="%s"'%user.__uri__, res.text)

    def test_module_validate(self):
        from ptah_crowd.views import CrowdApplicationView

        mod = self._make_mod()
        user = self._make_user()
        uri = user.__uri__

        props = ptah_crowd.get_properties(uri)
        props.validated = False

        form = CrowdApplicationView(mod, DummyRequest(
                POST=MultiDict((('uid', uri),
                                ('validate', 'validate')))))

        form.csrf = False
        form.update()
        transaction.commit()

        self.assertIn('Selected accounts have been validated.',
                      ptah.render_messages(form.request))
        props = ptah_crowd.get_properties(uri)
        self.assertTrue(props.validated)

    def test_module_suspend(self):
        from ptah_crowd.views import CrowdApplicationView

        mod = self._make_mod()
        user = self._make_user()
        uri = user.__uri__

        props = ptah_crowd.get_properties(uri)
        props.suspended = False

        form = CrowdApplicationView(mod, DummyRequest(
                POST=MultiDict((('uid', uri),
                                ('suspend', 'suspend')))))
        form.request.POST[form.csrfname] = form.request.session.get_csrf_token()
        form.update()

        self.assertIn('Selected accounts have been suspended.',
                      ptah.render_messages(form.request))
        transaction.commit()
        props = ptah_crowd.get_properties(uri)
        self.assertTrue(props.suspended)

    def test_module_activate(self):
        from ptah_crowd.views import CrowdApplicationView

        mod = self._make_mod()
        user = self._make_user()
        uri = user.__uri__

        props = ptah_crowd.get_properties(uri)
        props.suspended = True

        form = CrowdApplicationView(mod, DummyRequest(
                POST=MultiDict((('uid', uri),
                                ('activate', 'activate')))))

        form.csrf = False
        form.update()
        transaction.commit()

        self.assertIn('Selected accounts have been activated.',
                      ptah.render_messages(form.request))
        props = ptah_crowd.get_properties(uri)
        self.assertFalse(props.suspended)

    def test_module_remove(self):
        from ptah_crowd.views import CrowdApplicationView

        mod = self._make_mod()
        user = self._make_user()
        uri = user.__uri__

        props = ptah_crowd.get_properties(uri)
        props.suspended = False

        form = CrowdApplicationView(mod, DummyRequest(
                POST=MultiDict((('uid', uri),
                                ('remove', 'remove')))))
        form.update()

        self.assertIn('Selected accounts have been removed.',
                      ptah.render_messages(form.request))
        transaction.commit()

        user = ptah.resolve(user.__uri__)
        self.assertIsNone(user)



import transaction
import ptah, ptah_crowd
from ptah import form
from ptah.testing import PtahTestCase
from pyramid.testing import DummyRequest


class TestCheckLogin(PtahTestCase):

    def test_check_login(self):
        from ptah_crowd import login
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login', email='email')
        CrowdFactory().add(user)

        request = DummyRequest()

        self.assertRaises(
            form.Invalid, ptah_crowd.checkLoginValidator, None, 'login')

        class Field(object):
            """ """

        field = Field()
        field.value = 'login'
        ptah_crowd.checkLoginValidator(field, 'login')

        field.value = 'other-login'
        self.assertRaises(
            form.Invalid, ptah_crowd.checkLoginValidator, field, 'login')

    def test_lower(self):
        from ptah_crowd.schemas import lower

        self.assertEqual(lower('Tttt'), 'tttt')
        self.assertEqual(lower('tttT'), 'tttt')
        self.assertEqual(lower(lower), lower)



import sys
import ptah
from pyramid.testing import DummyRequest
from pyramid.httpexceptions import HTTPFound, HTTPForbidden


class TestResetPassword(ptah.PtahTestCase):

    def setUp(self):
        super(TestResetPassword, self).setUp()

        self._app = self.config.make_wsgi_app()

    def test_resetpassword_cancel(self):
        from ptah_crowd.resetpassword import ResetPassword
        request = DummyRequest(
            POST={'form.buttons.cancel': 'Cancel'})

        form = ResetPassword(None, request)
        res = form.update()

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], 'http://example.com')

    def test_resetpassword_required(self):
        from ptah_crowd.resetpassword import ResetPassword
        request = DummyRequest(
            POST={'form.buttons.reset': 'Reset'})

        form = ResetPassword(None, request)
        form.update()

        msg = ptah.render_messages(request)
        self.assertIn("System can't restore password for this user.", msg)

    def test_resetpassword(self):
        from ptah_crowd.resetpassword import ResetPassword
        from ptah_crowd.resetpassword import ResetPasswordTemplate
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login', email='email')
        CrowdFactory().add(user)

        data = [1, None]
        def send(self):
            data[0] = 2
            data[1] = self.passcode

        ResetPasswordTemplate.send = send

        request = DummyRequest(
            POST={'login': 'login',
                  'form.buttons.reset': 'Reset'})

        form = ResetPassword(None, request)
        res = form.update()

        msg = ptah.render_messages(request)
        self.assertIn("Password reseting process has been initiated.", msg)

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], 'http://example.com')
        self.assertEqual(data[0], 2)

        principal = ptah.pwd_tool.get_principal(data[1])
        self.assertEqual(principal.name, 'name')
        self.assertEqual(principal.login, 'login')

        del ResetPasswordTemplate.send

    def test_resetpassword_form_unknown_passcode(self):
        from ptah_crowd.resetpassword import ResetPasswordForm

        request = DummyRequest(subpath=('unknown',))

        form = ResetPasswordForm(None, request)
        res = form.update()

        msg = ptah.render_messages(request)
        self.assertIn("Passcode is invalid.", msg)
        self.assertEqual(
            res.headers['location'], 'http://example.com/resetpassword.html')

    def test_resetpassword_form_update(self):
        from ptah_crowd.resetpassword import ResetPasswordForm
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login', email='email')
        CrowdFactory().add(user)

        passcode = ptah.pwd_tool.generate_passcode(user)

        request = DummyRequest(subpath=(passcode,))

        form = ResetPasswordForm(None, request)
        form.update()

        self.assertEqual(form.title, user.name)
        self.assertEqual(form.passcode, passcode)

    def test_resetpassword_form_change_errors(self):
        from ptah_crowd.resetpassword import ResetPasswordForm
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login', email='email')
        CrowdFactory().add(user)

        passcode = ptah.pwd_tool.generate_passcode(user)

        request = DummyRequest(
            subpath=(passcode,),
            POST = {'password': '12345', 'confirm_password': '123456',
                    'form.buttons.change': 'Change'})
        request.environ['HTTP_HOST'] = 'example.com'

        form = ResetPasswordForm(None, request)
        form.update()

        msg = ptah.render_messages(request)
        self.assertIn("Please fix indicated errors.", msg)

    def test_resetpassword_form_change(self):
        from ptah_crowd.resetpassword import ResetPasswordForm
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login', email='email')
        CrowdFactory().add(user)

        passcode = ptah.pwd_tool.generate_passcode(user)

        request = DummyRequest(
            subpath=(passcode,),
            POST = {'password': '123456', 'confirm_password': '123456',
                    'form.buttons.change': 'Change'})
        request.environ['HTTP_HOST'] = 'example.com'

        form = ResetPasswordForm(None, request)
        res = form.update()

        msg = ptah.render_messages(request)
        self.assertIn("You have successfully changed your password.", msg)
        self.assertEqual(res.headers['location'], 'http://example.com')
        self.assertTrue(ptah.pwd_tool.check(user.password, '123456'))

    def test_resetpassword_template(self):
        from ptah_crowd.resetpassword import ResetPasswordTemplate
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login', email='email')
        CrowdFactory().add(user)

        request = DummyRequest()
        passcode = ptah.pwd_tool.generate_passcode(user)

        template = ResetPasswordTemplate(user, request)
        template.passcode = passcode

        template.update()
        text = template.render()

        self.assertIn(
            "Password reseting process has been initiated. You must visit link below to complete password reseting:", text)

        self.assertIn(
            "http://example.com/resetpassword.html/%s/"%passcode, text)



""" schemas """
from pyramid.compat import string_types

import ptah
from ptah import form
from ptah.password import passwordValidator

from ptah_crowd.settings import _


def lower(s):
    if isinstance(s, string_types):
        return s.lower()
    return s


def checkLoginValidator(node, login):
    if getattr(node, 'value', None) == login:
        return

    if ptah.auth_service.get_principal_bylogin(login) is not None:
        raise form.Invalid(node, _('Login already is in use.'))


RegistrationSchema = form.Fieldset(

    form.TextField(
        'name',
        title=_('Full Name'),
        description=_("e.g. John Smith. This is how users "
                      "on the site will identify you."),
        ),

    form.TextField(
        'login',
        title = _('E-mail/Login'),
        description = _('This is the username you will use to log in. '
                        'It must be an email address. <br /> Your email address '
                        'will not be displayed to any user or be shared with '
                        'anyone else.'),
        preparer = lower,
        validator = form.All(form.Email(), checkLoginValidator),
        )
    )


ResetPasswordSchema = form.Fieldset(

    form.TextField(
        'login',
        title = _('Login Name'),
        description = _('Login names are not case sensitive.'),
        missing = '',
        default = '')
    )


UserSchema = form.Fieldset(

    form.fields.TextField(
        'name',
        title=_('Full Name'),
        description=_("e.g. John Smith. This is how users "
                      "on the site will identify you."),
        ),

    form.fields.TextField(
        'login',
        title = _('E-mail/Login'),
        description=_('This is the username you will use to log in. '
                      'It must be an email address. <br /> Your email address '
                      'will not be displayed to any user or be shared with '
                      'anyone else.'),
        preparer = lower,
        validator = form.All(form.Email(), checkLoginValidator),
        ),

    form.fields.TextField(
        'password',
        title = _('Password'),
        description = _('Enter password. '\
                        'No spaces or special characters, should contain '\
                        'digits and letters in mixed case.'),
        validator = passwordValidator),

    form.fields.BoolField(
        'validated',
        title = _('Validated'),
        default = True,
        ),

    form.fields.BoolField(
        'suspended',
        title = _('Suspended'),
        default = False,
        ),

    )


ManagerChangePasswordSchema = form.Fieldset(

    form.PasswordField(
        'password',
        title = _('New password'),
        description = _('Enter new password. '\
                        'No spaces or special characters, should contain '\
                        'digits and letters in mixed case.'),
        validator = passwordValidator)
    )



""" account validation/suspending """
from datetime import timedelta, datetime
from pyramid.view import view_config
from pyramid.compat import text_type
from pyramid.security import remember
from pyramid.httpexceptions import HTTPFound

import ptah
from ptah import mail

from ptah_crowd.settings import CFG_ID_CROWD
from ptah_crowd.memberprops import get_properties, query_properties

TOKEN_TYPE = ptah.token.TokenType(
    'cd51f14e9b2842608ccadf1a240046c1', timedelta(hours=24))


def initiate_email_validation(email, principal, request):
    """ initiate principal email validation """
    t = ptah.token.service.generate(TOKEN_TYPE, principal.__uri__)
    template = ValidationTemplate(principal, request, email=email, token = t)
    template.send()


@ptah.auth_checker
def validationAndSuspendedChecker(info):
    props = get_properties(info.__uri__)
    if props.suspended:
        info.message = 'Account is suspended.'
        info.arguments['suspended'] = True
        return False

    if props.validated:
        return True

    CROWD = ptah.get_settings(CFG_ID_CROWD)
    if not CROWD['validation']:
        return True

    if CROWD['allow-unvalidated'] or props.validated:
        return True

    info.message = 'Account is not validated.'
    info.arguments['validation'] = False
    return False


@ptah.subscriber(ptah.events.PrincipalRegisteredEvent)
def principalRegistered(ev):
    props = get_properties(ev.principal.__uri__)
    props.joined = datetime.now()

    cfg = ptah.get_settings(CFG_ID_CROWD)
    if not cfg['validation']:
        props.validated = True


@ptah.subscriber(ptah.events.PrincipalAddedEvent)
def principalAdded(ev):
    props = get_properties(ev.principal.__uri__)
    props.joined = datetime.now()
    props.validated = True


class ValidationTemplate(mail.MailTemplate):

    subject = 'Activate Your Account'
    template = 'ptah_crowd:templates/validate_email.txt'

    def update(self):
        super(ValidationTemplate, self).update()

        self.url = '%s/validateaccount.html?token=%s'%(
            self.request.application_url, self.token)

        principal = self.context
        self.to_address = mail.formataddr((principal.name, self.email))


@view_config(route_name='ptah-principal-validate')
def validate(request):
    """Validate account"""
    t = request.GET.get('token')

    data = ptah.token.service.get(t)
    if data is not None:
        props = query_properties(data)
        if props is not None:
            props.validated = True
            ptah.token.service.remove(t)
            ptah.add_message(request,"Account has been successfully validated.")

            request.registry.notify(
                ptah.events.PrincipalValidatedEvent(ptah.resolve(props.uri)))

            headers = remember(request, props.uri)
            return HTTPFound(location=request.application_url, headers=headers)

    ptah.add_message(request, "Can't validate email address.", 'warning')
    return HTTPFound(location=request.application_url)



import transaction
import ptah
from ptah import config
from ptah.testing import PtahTestCase
from pyramid.testing import DummyRequest
from pyramid.httpexceptions import HTTPFound, HTTPForbidden

import ptah_crowd


class TestJoin(PtahTestCase):

    def test_join_auth(self):
        from ptah_crowd.registration import Registration

        request = DummyRequest()
        ptah.auth_service.set_userid('test')

        form = Registration(None, request)
        res = form.update()

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(
            res.headers['location'], 'http://example.com')

    def test_join_disabled(self):
        from ptah_crowd.registration import Registration

        cfg = ptah.get_settings(ptah_crowd.CFG_ID_CROWD)
        cfg['join'] = False

        request = DummyRequest()
        form = Registration(None, request)
        res = form.update()
        self.assertIsInstance(res, HTTPForbidden)

    def test_join_error(self):
        from ptah_crowd.registration import Registration
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login', email='email')
        CrowdFactory().add(user)

        uri = user.__uri__

        request = DummyRequest(
            POST = {'name': 'Test user',
                    'login': 'custom login',
                    'password': '12345',
                    'confirm_password': '123456'})

        form = Registration(None, request)
        form.update()

        data, errors = form.extract()
        self.assertEqual(len(errors), 1)
        self.assertEqual(errors[0].msg[0], 'Invalid email address')

        form.register_handler()
        self.assertIn('Please fix indicated errors.',
                      ptah.render_messages(request))

        request = DummyRequest(
            POST = {'name': 'Test user',
                    'login': 'test@example.com',
                    'password': '12345',
                    'confirm_password': '12345'})
        form = Registration(None, request)
        form.update()
        data, errors = form.extract()
        self.assertEqual(len(errors), 0)

    def test_join(self):
        import ptah
        from ptah_crowd.registration import Registration
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login', email='email')
        CrowdFactory().add(user)

        uri = user.__uri__

        class Stub(object):
            status = ''
            def send(self, frm, to, msg):
                Stub.status = 'Email has been sended'

        MAIL = ptah.get_settings(ptah.CFG_ID_PTAH)
        MAIL['Mailer'] = Stub()

        request = DummyRequest(
            POST = {'name': 'Test user',
                    'login': 'test@example.com',
                    'password': '12345',
                    'confirm_password': '12345'})
        request.environ['HTTP_HOST'] = 'example.com'

        form = Registration(None, request)
        form.update()
        res = form.register_handler()

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'],
                         'http://example.com/login-success.html')

        user = ptah.auth_service.get_principal_bylogin('test@example.com')
        self.assertIsInstance(user, CrowdUser)
        self.assertEqual(user.name, 'Test user')

    def test_join_unvalidated(self):
        from ptah_crowd.registration import Registration
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login', email='email')
        CrowdFactory().add(user)

        uri = user.__uri__

        class Stub(object):
            status = ''
            def send(self, frm, to, msg):
                Stub.status = 'Email has been sended'

        MAIL = ptah.get_settings(ptah.CFG_ID_PTAH)
        MAIL['Mailer'] = Stub()

        CROWD = ptah.get_settings(ptah_crowd.CFG_ID_CROWD)
        CROWD['allow-unvalidated'] = False

        request = DummyRequest(
            POST = {'name': 'Test user',
                    'login': 'test@example.com',
                    'password': '12345',
                    'confirm_password': '12345'})
        form = Registration(None, request)
        form.update()
        res = form.register_handler()

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], 'http://example.com')

        user = ptah.auth_service.get_principal_bylogin('test@example.com')
        self.assertIsInstance(user, CrowdUser)
        self.assertEqual(user.name, 'Test user')

        self.assertIn('Validation email has been sent.',
                      ptah.render_messages(request))



import sqlalchemy as sqla
from pyramid.view import view_config
from pyramid.httpexceptions import HTTPFound

import ptah
import ptah_crowd
from ptah import form
from ptah_crowd.settings import _
from ptah_crowd.module import CrowdModule
from ptah_crowd.provider import CrowdUser, CrowdFactory
from ptah_crowd.memberprops import MemberProperties


@view_config(
    context=CrowdModule,
    wrapper=ptah.wrap_layout(),
    renderer='ptah_crowd:templates/users.pt')

class CrowdApplicationView(form.Form):
    __doc__ = 'List/search users view'

    csrf = True
    fields = form.Fieldset(
        form.TextField(
            'term',
            title = _('Search term'),
            description = _('Ptah searches users by login and email'),
            missing = '',
            default = '')
        )

    users = None
    page = ptah.Pagination(15)

    def form_content(self):
        return {'term': self.request.session.get('ptah-search-term', '')}

    def update(self):
        super(CrowdApplicationView, self).update()

        request = self.request
        self.manage_url = ptah.manage.get_manage_url(self.request)

        Session = ptah.get_session()
        uids = request.POST.getall('uid')

        if 'activate' in request.POST and uids:
            Session.query(MemberProperties)\
                .filter(MemberProperties.uri.in_(uids))\
                .update({'suspended': False}, False)
            self.message("Selected accounts have been activated.", 'info')

        if 'suspend' in request.POST and uids:
            Session.query(MemberProperties).filter(
                MemberProperties.uri.in_(uids))\
                .update({'suspended': True}, False)
            self.message("Selected accounts have been suspended.", 'info')

        if 'validate' in request.POST and uids:
            Session.query(MemberProperties).filter(
                MemberProperties.uri.in_(uids))\
                .update({'validated': True}, False)
            self.message("Selected accounts have been validated.", 'info')

        if 'remove' in request.POST and uids:
            crowd = CrowdFactory()
            for user in Session.query(CrowdUser).filter(
                CrowdUser.__uri__.in_(uids)):
                Session.delete(ptah_crowd.get_properties(user.__uri__))
                del crowd[user.__name__]
            self.message("Selected accounts have been removed.", 'info')

        term = request.session.get('ptah-search-term', '')
        if term:
            self.users = Session.query(CrowdUser) \
                .filter(sqla.sql.or_(
                    CrowdUser.email.contains('%%%s%%'%term),
                    CrowdUser.title.contains('%%%s%%'%term)))\
                .order_by(sqla.sql.asc('name')).all()
        else:
            self.size = Session.query(CrowdUser).count()

            try:
                current = int(request.params.get('batch', None))
                if not current:
                    current = 1

                request.session['crowd-current-batch'] = current
            except:
                current = request.session.get('crowd-current-batch')
                if not current:
                    current = 1

            self.current = current

            self.pages, self.prev, self.next = self.page(self.size,self.current)

            offset, limit = self.page.offset(current)
            self.users = Session.query(CrowdUser)\
                    .offset(offset).limit(limit).all()

    @form.button(_('Search'), actype=form.AC_PRIMARY)
    def search(self):
        data, error = self.extract()

        if not data['term']:
            self.message('Please specify search term', 'warning')
            return

        self.request.session['ptah-search-term'] = data['term']

    @form.button(_('Clear term'), name='clear')
    def clear(self):
        if 'ptah-search-term' in self.request.session:
            del self.request.session['ptah-search-term']



import transaction
import ptah
from ptah import config
from ptah.testing import PtahTestCase
from pyramid.testing import DummyRequest
from pyramid.httpexceptions import HTTPException, HTTPFound

import ptah_crowd


class TestSuspended(PtahTestCase):

    def test_suspended_anon(self):
        from ptah_crowd import login

        request = DummyRequest()
        view = login.LoginSuspended(None, request)
        res = view.update()
        self.assertIsInstance(res, HTTPFound)

    def test_suspended_not(self):
        from ptah_crowd import login
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login',
                         email='email', password='{plain}12345')
        CrowdFactory().add(user)

        uri = user.__uri__
        props = ptah_crowd.get_properties(uri)
        props.suspended = False

        request = DummyRequest()
        ptah.auth_service.set_userid(uri)

        request = DummyRequest()
        res = self.render_route_view(None, request, 'ptah-login-suspended')
        self.assertIsInstance(res, HTTPFound)

    def test_suspended(self):
        from ptah_crowd import login
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login',
                         email='email', password='{plain}12345')
        CrowdFactory().add(user)

        uri = user.__uri__

        props = ptah_crowd.get_properties(uri)
        props.suspended = True

        ptah.auth_service.set_userid(user.__uri__)

        request = DummyRequest()
        res = self.render_route_view(None, request, 'ptah-login-suspended')

        self.assertIn('Your account is suspended', res.text)


class TestLogout(PtahTestCase):

    def test_logout_anon(self):
        from ptah_crowd import login

        request = DummyRequest()
        res = login.logout(request)
        self.assertIsInstance(res, HTTPFound)

    def test_logout(self):
        from ptah_crowd import login
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login',
                         email='email', password='{plain}12345')
        CrowdFactory().add(user)

        uri = user.__uri__

        request = DummyRequest()
        request.environ['HTTP_HOST'] = 'example.com'
        ptah.auth_service.set_userid(uri)

        res = login.logout(request)
        self.assertIsInstance(res, HTTPFound)
        self.assertIsNone(ptah.auth_service.get_userid())


class TestLogoutSuccess(PtahTestCase):

    def test_login_success_anon(self):
        from ptah_crowd import login

        request = DummyRequest()
        request.environ['HTTP_HOST'] = 'example.com'

        res = self.render_route_view(None, request, 'ptah-login-success')

        self.assertEqual(
            res.headers['location'], 'http://example.com/login.html')

    def test_login_success(self):
        from ptah_crowd import login
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login',
                         email='email', password='{plain}12345')
        CrowdFactory().add(user)

        uri = user.__uri__

        request = DummyRequest()
        ptah.auth_service.set_userid(uri)

        res = self.render_route_view(None, request, 'ptah-login-success')
        self.assertIn('You are now logged in', res.text)


class TestLogin(PtahTestCase):

    def test_login_auth(self):
        from ptah_crowd import login

        request = DummyRequest()

        ptah.auth_service.set_userid('test')

        res = login.LoginForm(None, request)()

        self.assertEqual(res.status, '302 Found')
        self.assertEqual(res.headers['location'],
                         'http://example.com/login-success.html')

    def test_login_update(self):
        from ptah_crowd import login

        request = DummyRequest()

        cfg = ptah.get_settings(ptah_crowd.CFG_ID_CROWD)
        cfg['join'] = False
        cfg['joinurl'] = 'http://test/login.html'

        form = login.LoginForm(None, request)
        form.update()
        self.assertFalse(form.join)
        self.assertEqual(form.joinurl, 'http://test/login.html')

        res = login.LoginForm(None, request)()
        self.assertNotIn('head over to the registration form', res.text)

    def test_login_update_join(self):
        from ptah_crowd import login

        request = DummyRequest()
        request.registry = self.registry

        cfg = ptah.get_settings(ptah_crowd.CFG_ID_CROWD, self.registry)
        cfg['join'] = True
        cfg['joinurl'] = ''

        form = login.LoginForm(None, request)
        form.update()
        self.assertTrue(form.join)
        self.assertEqual(form.joinurl, 'http://example.com/join.html')

        #self.assertIn('head over to the registration form', res.text)

    def test_login(self):
        from ptah_crowd import login
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login',
                         email='email', password='{plain}12345')
        CrowdFactory().add(user)

        uri = user.__uri__

        request = DummyRequest()

        form = login.LoginForm(None, request)
        form.update()
        data, errors = form.extract()
        self.assertEqual(len(errors), 2)

        form.handleLogin()
        self.assertIn('Please fix indicated errors.',
                      ptah.render_messages(request))

        request = DummyRequest(
            POST={'login': 'login', 'password': '12345'})
        request.environ['HTTP_HOST'] = 'example.com'

        form = login.LoginForm(None, request)
        form.update()
        res = form.handleLogin()

        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'],
                         'http://example.com/login-success.html')

    def test_login_came_from(self):
        from ptah_crowd import login
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login',
                         email='email', password='{plain}12345')
        CrowdFactory().add(user)

        uri = user.__uri__

        request = DummyRequest(
            POST={'login': 'login', 'password': '12345'},
            GET={'came_from': 'http://example.com/ptah-manage/'})
        request.environ['HTTP_HOST'] = 'example.com'

        form = login.LoginForm(None, request)
        form.update()
        res = form.handleLogin()

        self.assertEqual(res.headers['location'],
                         'http://example.com/ptah-manage/')

    def test_login_wrong_login(self):
        from ptah_crowd import login
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login',
                         email='email', password='{plain}12345')
        CrowdFactory().add(user)

        uri = user.__uri__

        request = DummyRequest(
            POST={'login': 'login1', 'password': '123456'})
        request.environ['HTTP_HOST'] = 'example.com'

        form = login.LoginForm(None, request)
        form.update()
        form.handleLogin()

        self.assertIn('You enter wrong login or password',
                      ptah.render_messages(request))

    def test_login_unvalidated(self):
        from ptah_crowd import login
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login',
                         email='email', password='{plain}12345')
        CrowdFactory().add(user)

        uri = user.__uri__
        ptah_crowd.get_properties(uri).validated = False

        cfg = ptah.get_settings(ptah_crowd.CFG_ID_CROWD)
        cfg['allow-unvalidated'] = False

        request = DummyRequest(
            POST={'login': 'login', 'password': '12345'})
        request.environ['HTTP_HOST'] = 'example.com'

        form = login.LoginForm(None, request)
        form.update()
        form.handleLogin()

        self.assertIn('Account is not validated.',ptah.render_messages(request))

    def test_login_suspended(self):
        from ptah_crowd import login
        from ptah_crowd.provider import CrowdUser, CrowdFactory

        user = CrowdUser(title='name', login='login',
                         email='email', password='{plain}12345')
        CrowdFactory().add(user)

        uri = user.__uri__
        user.properties.suspended = True

        request = DummyRequest(
            POST={'login': 'login', 'password': '12345'})
        request.environ['HTTP_HOST'] = 'example.com'

        form = login.LoginForm(None, request)
        form.update()
        res = form.handleLogin()

        self.assertEqual(res.headers['location'],
                         'http://example.com/login-suspended.html')



""" role """
import transaction
import ptah, ptah_crowd
from ptah import config
from ptah.testing import PtahTestCase


class TestCrowdApplication(PtahTestCase):

    def test_crowd_factory(self):
        import ptah_crowd

        factory = ptah_crowd.CrowdFactory()
        self.assertEqual(factory.__name__, ptah_crowd.CROWD_APP_ID)
        self.assertIsInstance(factory, ptah_crowd.CrowdApplication)

    def test_crowd_app_add_user(self):
        import ptah_crowd

        factory = ptah_crowd.CrowdFactory()

        user = ptah_crowd.CrowdUser()
        factory.add(user)

        self.assertEqual(user.__name__, str(user.__id__))

    def test_crowd_allowed(self):
        from ptah_crowd.provider import get_allowed_content_types

        self.assertEqual(get_allowed_content_types(None), ('ptah-crowd-user',))


class TestProvider(PtahTestCase):

    def test_authenticate(self):
        from ptah_crowd.provider import \
             CrowdAuthProvider, CrowdFactory, CrowdUser

        provider = CrowdAuthProvider()

        self.assertFalse(
            provider.authenticate(
                {'login': 'test', 'password': '12345'}))

        user = CrowdUser(title='test',
                         login='test',
                         email='test@ptahproject.org',
                         password=ptah.pwd_tool.encode('12345'))
        factory = CrowdFactory()
        factory.add(user)

        self.assertTrue(
            provider.authenticate(
                {'login': 'test', 'password': '12345'}))

        self.assertFalse(
            provider.authenticate(
                {'login': 'test', 'password': '56789'}))


    def test_get_bylogin(self):
        from ptah_crowd.provider import \
             CrowdAuthProvider, CrowdFactory, CrowdUser

        provider = CrowdAuthProvider()
        self.assertIsNone(provider.get_principal_bylogin('test'))

        user = CrowdUser(title='test', login='test',
                         email='test@ptahproject.org',
                         password=ptah.pwd_tool.encode('12345'))
        CrowdFactory().add(user)

        user = provider.get_principal_bylogin('test')
        self.assertIsInstance(user, CrowdUser)
        self.assertEqual(user.login, 'test')

    def test_crowd_user_ctor(self):
        from ptah_crowd.provider import CrowdUser

        user = CrowdUser(title='user-name', login='user-login',
                         email='user-email', password='passwd')

        self.assertEqual(user.title, 'user-name')
        self.assertEqual(user.login, 'user-login')
        self.assertEqual(user.email, 'user-email')
        self.assertEqual(user.password, 'passwd')
        self.assertTrue(user.__uri__.startswith('cms-ptah-crowd-user'))
        self.assertEqual(str(user), 'user-name')
        self.assertEqual(repr(user), 'CrowdUser<%s:%s>'%(
            user.name, user.__uri__))

    def test_crowd_user_change_password(self):
        from ptah_crowd.provider import CrowdUser, CrowdAuthProvider

        user = CrowdUser(title='user-name', login='user-login',
                         email='user-email', password='passwd')

        CrowdAuthProvider.change_password(user, '123456')
        self.assertEqual(user.password, '123456')

    def test_crowd_user_change_search(self):
        from ptah_crowd.provider import \
             CrowdUser, CrowdFactory, CrowdAuthProvider

        user = CrowdUser(title='user-name', login='user-login',
                         email='user-email', password='passwd')
        uri = user.__uri__

        CrowdFactory().add(user)

        users = list(CrowdAuthProvider.search('user'))
        self.assertEqual(len(users), 1)
        self.assertEqual(users[0].__uri__, uri)

        users = list(CrowdAuthProvider.search('email'))
        self.assertEqual(len(users), 1)
        self.assertEqual(users[0].__uri__, uri)


class TestPasswordChanger(PtahTestCase):

    def test_password_changer(self):
        from ptah_crowd.provider import CrowdUser

        app = self.config.make_wsgi_app()

        user = CrowdUser(title='user-name', login='user-login',
                         email='user-email', password='passwd')

        self.assertTrue(ptah.pwd_tool.can_change_password(user))



""" reset password form """
import ptah
from datetime import datetime
from pyramid import security
from pyramid.view import view_config
from pyramid.httpexceptions import HTTPFound

from ptah.password import PasswordSchema
from ptah.events import ResetPasswordInitiatedEvent
from ptah.events import PrincipalPasswordChangedEvent

from ptah_crowd.settings import _


@view_config(
    route_name='ptah-resetpassword',
    wrapper=ptah.wrap_layout('ptah-page'),
    renderer='ptah_crowd:templates/resetpassword.pt')

class ResetPassword(ptah.form.Form):

    fields = ptah.form.Fieldset(
        ptah.form.FieldFactory(
            'text',
            'login',
            title = _('Login Name'),
            description = _('Login names are not case sensitive.'),
            missing = '',
            default = ''))

    def form_content(self):
        return {'login': self.request.params.get('login', '')}

    def update(self):
        cfg = ptah.get_settings(ptah.CFG_ID_PTAH, self.request.registry)
        self.from_name = cfg['email_from_name']
        self.from_address = cfg['email_from_address']

        return super(ResetPassword, self).update()

    @ptah.form.button(_('Start password reset'),
                      name='reset', actype=ptah.form.AC_PRIMARY)
    def reset(self):
        request = self.request
        registry = request.registry
        data, errors = self.extract()

        login = data.get('login')
        if login:
            principal = ptah.auth_service.get_principal_bylogin(login)

            if principal is not None and \
                   ptah.pwd_tool.can_change_password(principal):
                passcode = ptah.pwd_tool.generate_passcode(principal)

                template = ResetPasswordTemplate(
                    principal, request, passcode = passcode)
                template.send()

                self.request.registry.notify(
                    ResetPasswordInitiatedEvent(principal))

                self.message(_('Password reseting process has been initiated. '
                               'Check your email for futher instructions.'))
                return HTTPFound(location=request.application_url)

        self.message(_("System can't restore password for this user."))

    @ptah.form.button(_('Cancel'))
    def cancel(self):
        return HTTPFound(location=self.request.application_url)


@view_config(
    route_name='ptah-resetpassword-form',
    wrapper=ptah.wrap_layout('ptah-page'),
    renderer='ptah_crowd:templates/resetpasswordform.pt')

class ResetPasswordForm(ptah.form.Form):

    fields = PasswordSchema

    def update(self):
        request = self.request

        passcode = request.subpath[0]
        self.principal = principal = ptah.pwd_tool.get_principal(passcode)

        if principal is not None and \
               ptah.pwd_tool.can_change_password(principal):
            self.passcode = passcode
            self.title = principal.name or principal.login
        else:
            self.message(_("Passcode is invalid."), 'warning')
            return HTTPFound(
                location='%s/resetpassword.html'%request.application_url)

        return super(ResetPasswordForm, self).update()

    @ptah.form.button(_("Change password"),
                      name='change', actype=ptah.form.AC_PRIMARY)
    def changePassword(self):
        data, errors = self.extract()

        if errors:
            self.message(errors, 'form-error')
        else:
            principal = ptah.pwd_tool.get_principal(self.passcode)
            ptah.pwd_tool.change_password(self.passcode, data['password'])

            self.request.registry.notify(
                PrincipalPasswordChangedEvent(principal))

            # check if principal can be authenticated
            info = ptah.auth_service.authenticate_principal(principal)

            headers = []
            if info.status:
                headers = security.remember(self.request, info.__uri__)

            self.message(
                _('You have successfully changed your password.'), 'success')
            return HTTPFound(
                headers = headers,
                location = self.request.application_url)


class ResetPasswordTemplate(ptah.mail.MailTemplate):

    subject = 'Password Reset Confirmation'
    template = 'ptah_crowd:templates/resetpasswordmail.pt'

    def update(self):
        super(ResetPasswordTemplate, self).update()

        request = self.request

        self.date = datetime.now()

        remoteAddr = request.get('REMOTE_ADDR', '')
        forwardedFor = request.get('HTTP_X_FORWARDED_FOR', None)

        self.from_ip = (forwardedFor and '%s/%s' %
                        (remoteAddr, forwardedFor) or remoteAddr)

        self.url = '%s/resetpassword.html/%s/'%(
            request.application_url, self.passcode)

        info = self.context

        self.to_address = ptah.mail.formataddr((info.name, info.login))



import os
import sys
import logging
import multiprocessing # atexit exception
from setuptools import setup, find_packages

version='0.1'

install_requires = ['setuptools',
                    'ptah >= 0.2',
                    ]
tests_require = ['nose']

def read(f):
    return open(os.path.join(os.path.dirname(__file__), f)).read().strip()


setup(name='ptah_crowd',
      version=version,
      description=('User management for Ptah.'),
      long_description='\n\n'.join((read('README.rst'), read('CHANGES.txt'))),
      classifiers=[
          "Intended Audience :: Developers",
          "Programming Language :: Python",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3",
          "Programming Language :: Python :: 3.2",
          "Programming Language :: Python :: Implementation :: CPython",
          "Framework :: Pylons",
          "Topic :: Internet :: WWW/HTTP",
          'Topic :: Internet :: WWW/HTTP :: WSGI'],
      author='Ptah Project',
      author_email='ptahproject@googlegroups.com',
      url='https://github.com/ptahproject/ptah_crowd/',
      license='BSD-derived',
      packages=find_packages(),
      install_requires = install_requires,
      tests_require = tests_require,
      test_suite = 'nose.collector',
      include_package_data = True,
      zip_safe = False,
      )



""" crowd module """
from pyramid.interfaces import IRequest, IRouteRequest

import ptah
import ptah_crowd


@ptah.manage.module('crowd')
class CrowdModule(ptah.manage.PtahModule):
    __doc__ = 'Default user management. Create, edit, and activate users.'

    title = 'User management'

    def __getitem__(self, key):
        crowd = ptah_crowd.CrowdFactory()
        user = crowd.get(key)
        if user is not None:
            user.__parent__ = self
            user.__resource_url__ = None

            request = self.request
            request.request_iface = request.registry.getUtility(
                IRouteRequest, name=ptah_crowd.CROWD_APP_ID)

            return user

        raise KeyError(key)



""" user registration form """
from pyramid import security
from pyramid.view import view_config
from pyramid.httpexceptions import HTTPFound, HTTPForbidden

import ptah
from ptah import config, form
from ptah.password import PasswordSchema
from ptah.events import PrincipalRegisteredEvent

from ptah_crowd.provider import CrowdFactory
from ptah_crowd.settings import _, CFG_ID_CROWD
from ptah_crowd.schemas import RegistrationSchema
from ptah_crowd.validation import initiate_email_validation


@view_config(
    route_name='ptah-join',
    wrapper=ptah.wrap_layout('ptah-page'))

class Registration(form.Form):
    """ Ptah crowd registration form """

    label = _("Registration")
    fields = form.Fieldset(RegistrationSchema, PasswordSchema)
    autocomplete = 'off'

    def update(self):
        uri = ptah.auth_service.get_userid()
        if uri is not None:
            return HTTPFound(location = self.request.application_url)

        self.cfg = ptah.get_settings(CFG_ID_CROWD, self.request.registry)
        if not self.cfg['join'] or not self.cfg['type']:
            return HTTPForbidden('Site registraion is disabled.')

        return super(Registration, self).update()

    def create(self, data):
        tp = self.cfg['type']
        if not tp.startswith('cms-type:'):
            tp = 'cms-type:{0}'.format(tp)

        tinfo = ptah.resolve(tp)

        # create user
        user = tinfo.create(
            title=data['name'], login=data['login'], email=data['login'])

        # set password
        user.password = ptah.pwd_tool.encode(data['password'])
        CrowdFactory().add(user)

        return user

    @form.button(_("Register"), actype=form.AC_PRIMARY)
    def register_handler(self):
        data, errors = self.extract()
        if errors:
            self.message(errors, 'form-error')
            return

        user = self.create(data)
        self.request.registry.notify(PrincipalRegisteredEvent(user))

        # validation
        if self.cfg['validation']:
            initiate_email_validation(user.email, user, self.request)
            self.message('Validation email has been sent.')
            if not self.cfg['allow-unvalidated']:
                return HTTPFound(location=self.request.application_url)

        # authenticate
        info = ptah.auth_service.authenticate(
            {'login': user.login, 'password': user.password})
        if info.status:
            headers = security.remember(self.request, info.__uri__)
            return HTTPFound(
                location='%s/login-success.html'%self.request.application_url,
                headers = headers)
        else:
            self.message(info.message) # pragma: no cover
